@@ -118,10 +118,26 @@ class BlogPostComment(RESTFrameworkModel):
     blog_post = models.ForeignKey(BlogPost)
 
 
+class Album(RESTFrameworkModel):
+    title = models.CharField(max_length=100, unique=True)
+
+
+class Photo(RESTFrameworkModel):
+    description = models.TextField()
+    album = models.ForeignKey(Album)
+
+
 class Person(RESTFrameworkModel):
     name = models.CharField(max_length=10)
     age = models.IntegerField(null=True, blank=True)
 
+    @property
+    def info(self):
+        return {
+            'name': self.name,
+            'age': self.age,
+        }
+
 
 # Model for issue #324
 class BlankFieldModel(RESTFrameworkModel):
